;;; protocol.lisp --- Protocol for finding and reading gi repositories.
;;
;; Copyright (C) 2010, 2011 Jan Moringen
;;
;; Author: Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
;;
;; This Program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This Program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program. If not, see <http://www.gnu.org/licenses>.

(in-package :gi.repository)


;;; Locating Repositories
;;

(defvar *repository-searchpath*
  '((#p"/usr/share/gir-1.0/" :xml))
  "A list of searchpath entries of the form (DIRECTORY FORMAT) where
DIRECTORY is a pathname and FORMAT is a symbol designating a
repository format.")

(defun find-repository-location (name version)
  "Find a file containing GObject introspection information for the
package specified by NAME and VERSION. If such a file is found on the
search path specified by `*repository-searchpath*', three values are
returned:
1. A pathname of the found file
2. The version of the repository stored in the found file
3. The format of the repository stored in the found file
If no matching repository is found, an error is signaled."
  (iter (for (location format) in *repository-searchpath*)
	(let ((candidate (make-pathname
			  :name     (format nil "~a-~a" name version)
			  :type     "gir"
			  :defaults location)))
	  (when (probe-file candidate)
	    (return-from find-repository-location
	      (values candidate version format)))))
  (error "Could not find repository for ~a, version ~a"
	 name version))


;;;
;;

(defgeneric read-repository (source format)
  (:documentation
   "Read contents of repository designated by SOURCE given the
information is stored in way described by FORMAT."))

(defmethod read-repository ((source string) (format t))
  ""
  (read-repository (parse-namestring source) format))

(defmethod read-repository ((source pathname) (format t))
  ""
  (with-input-from-file (stream source)
    (read-repository stream format)))


;;;
;;

(defvar *repositories* (make-hash-table :test #'equal)
  "DOC")

(defun make-repository-key (name version)
  "DOC"
  (declare (inline make-repository-key))

  (list name version))

(defun find-repository (name version)
  "Try to find and return the repository identified by the pair (NAME
VERSION). If there is no such repository, return nil."
  (gethash (make-repository-key name version) *repositories*))

(defun (setf find-repository) (new-value name version)
  "DOC"
  (setf (gethash (make-repository-key name version) *repositories*)
	new-value))

(defun ensure-repository (name version)
  "Find or read and return the repository identified by the pair (NAME
VERSION)."
  (or (find-repository name version)
      (setf (find-repository name version)
	    (multiple-value-bind (pathname version format)
		(find-repository-location name version)
	      (declare (ignore version))
	      (read-repository pathname format)))))

(defun clear-repositories! ()
  (clrhash *repositories*))
